/** LPP Dir lib by Nanni */

/*----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#------  This File is Part Of : ----------------------------------------------------------------------------------------#
#------- _  -------------------  ______   _   --------------------------------------------------------------------------#
#------ | | ------------------- (_____ \ | |  --------------------------------------------------------------------------#
#------ | | ---  _   _   ____    _____) )| |  ____  _   _   ____   ____   ----------------------------------------------#
#------ | | --- | | | | / _  |  |  ____/ | | / _  || | | | / _  ) / ___)  ----------------------------------------------#
#------ | |_____| |_| |( ( | |  | |      | |( ( | || |_| |( (/ / | |  --------------------------------------------------#
#------ |_______)\____| \_||_|  |_|      |_| \_||_| \__  | \____)|_|  --------------------------------------------------#
#------------------------------------------------- (____/  -------------------------------------------------------------#
#------------------------   ______   _   -------------------------------------------------------------------------------#
#------------------------  (_____ \ | |  -------------------------------------------------------------------------------#
#------------------------   _____) )| | _   _   ___   ------------------------------------------------------------------#
#------------------------  |  ____/ | || | | | /___)  ------------------------------------------------------------------#
#------------------------  | |      | || |_| ||___ |  ------------------------------------------------------------------#
#------------------------  |_|      |_| \____|(___/   ------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#- Licensed under the GPL License --------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#- Copyright (c) Nanni <cusunanni@hotmail.it> --------------------------------------------------------------------------#
#- Copyright (c) Rinnegatamante <rinnegatamante@eternalongju2.com> -----------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#- Official Forum : http://rinnegatamante.eu/luaplayerplus/forum.php ---------------------------------------------------#
#- For help using LuaPlayerPlus, coding help, and other please visit : http://rinnegatamante.eu/luaplayerplus/forum.php #
#-----------------------------------------------------------------------------------------------------------------------#
#- Credits : -----------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#- Homemister for LPHM sourcecode --------------------------------------------------------------------------------------#
#- Zack & Shine for LP Euphoria sourcecode -----------------------------------------------------------------------------#
#- ab5000 for support on psp-ita.com -----------------------------------------------------------------------------------#
#- valantin for sceIoMvdir and sceIoCpdir improved functions------------------------------------------------------------#
#- Dark_AleX for usbdevice ---------------------------------------------------------------------------------------------#
#- VirtuosFlame & ColdBird for iso drivers and kuBridge ----------------------------------------------------------------#
#- sakya for Media Engine and OslibMod ---------------------------------------------------------------------------------#
#- Booster & silverspring for EEPROM write/read functions --------------------------------------------------------------#
#- Akind for RemoteJoyLite ---------------------------------------------------------------------------------------------#
#- cooleyes for mpeg4 lib ----------------------------------------------------------------------------------------------#
#- Arshia001 for PSPAALib ----------------------------------------------------------------------------------------------#
#- InsertWittyName & MK2k for PGE sourcecode ---------------------------------------------------------------------------#
#- Youresam for LUA BMPLib ---------------------------------------------------------------------------------------------#
#- Raphael for vram manager code ---------------------------------------------------------------------------------------#
#- Dynodzzo for LSD concepts -------------------------------------------------------------------------------------------#
#- ab_portugal for Image.negative function -----------------------------------------------------------------------------#
#- JiC for drawCircle function ----------------------------------------------------------------------------------------#
#- Rapper_skull & DarkGiovy for testing LuaPlayer Plus and coming up with some neat ideas for it. ----------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------*/

#ifndef __DIRL_H_
#define __DIRL_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "../Types.h"
#include <stdio.h>

/**
 * Enum for directory entry type.
 */
enum LPP_Dir_Entry_Type
{
    LPP_DIR_ENTRY_TYPE_FILE = 0,
    LPP_DIR_ENTRY_TYPE_DIR = 1
};

/**
 * A directory entry struct.
 */
typedef struct {
    char name[256];
    size_t size;
    u32 type;
} LPP_DirEntry;

typedef struct {
    int fd;
    u32 count;
    LPP_DirEntry *entries;
} LPP_Dir;

/**
 * Create a Directory.
 *
 * @param dir - path of directory to create.
 *
 */
extern int LPP_MkDir(const char *dir);

/**
 * Check if a file exists.
 *
 * @param Filename - the file to verify the existence.
 *
 */
extern unsigned char LPP_FileExists(const char *Filename);

/**
 * Remove a directory.
 */
void LPP_RemoveDir(const char *dir);

/**
 * Delete a file.
 */
void LPP_RemoveFile(const char *path);

int LPP_Chdir(const char *path);

int LPP_GetCwd(char *path, size_t size);

LPP_Dir *LPP_DirOpen(const char *path);

int LPP_DirRead(LPP_Dir *dir);

void LPP_DirClose(LPP_Dir *dir);

u8 LPP_DirExists(const char *path);

int LPP_DirRename(const char *oldname, const char *newname);

int LPP_FileRename(const char *oldname, const char *newname);

void LPP_MoveFile(const char *oldpath, const char *newpath);

void LPP_CopyFile(const char *oldpath, const char *newpath);

void LPP_CopyDir(const char *oldPath, const char *newPath);

void LPP_MoveDir(const char *oldPath, const char *newPath);

#ifdef __cplusplus
}
#endif

#endif /* __DIRL_H_ */
