/** LPP Timer lib by Nanni */

/*----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#------  This File is Part Of : ----------------------------------------------------------------------------------------#
#------- _  -------------------  ______   _   --------------------------------------------------------------------------#
#------ | | ------------------- (_____ \ | |  --------------------------------------------------------------------------#
#------ | | ---  _   _   ____    _____) )| |  ____  _   _   ____   ____   ----------------------------------------------#
#------ | | --- | | | | / _  |  |  ____/ | | / _  || | | | / _  ) / ___)  ----------------------------------------------#
#------ | |_____| |_| |( ( | |  | |      | |( ( | || |_| |( (/ / | |  --------------------------------------------------#
#------ |_______)\____| \_||_|  |_|      |_| \_||_| \__  | \____)|_|  --------------------------------------------------#
#------------------------------------------------- (____/  -------------------------------------------------------------#
#------------------------   ______   _   -------------------------------------------------------------------------------#
#------------------------  (_____ \ | |  -------------------------------------------------------------------------------#
#------------------------   _____) )| | _   _   ___   ------------------------------------------------------------------#
#------------------------  |  ____/ | || | | | /___)  ------------------------------------------------------------------#
#------------------------  | |      | || |_| ||___ |  ------------------------------------------------------------------#
#------------------------  |_|      |_| \____|(___/   ------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#- Licensed under the GPL License --------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#- Copyright (c) Nanni <cusunanni@hotmail.it> --------------------------------------------------------------------------#
#- Copyright (c) Rinnegatamante <rinnegatamante@eternalongju2.com> -----------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#- Official Forum : http://rinnegatamante.eu/luaplayerplus/forum.php ---------------------------------------------------#
#- For help using LuaPlayerPlus, coding help, and other please visit : http://rinnegatamante.eu/luaplayerplus/forum.php #
#-----------------------------------------------------------------------------------------------------------------------#
#- Credits : -----------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#- Homemister for LPHM sourcecode --------------------------------------------------------------------------------------#
#- Zack & Shine for LP Euphoria sourcecode -----------------------------------------------------------------------------#
#- ab5000 for support on psp-ita.com -----------------------------------------------------------------------------------#
#- valantin for sceIoMvdir and sceIoCpdir improved functions------------------------------------------------------------#
#- Dark_AleX for usbdevice ---------------------------------------------------------------------------------------------#
#- VirtuosFlame & ColdBird for iso drivers and kuBridge ----------------------------------------------------------------#
#- sakya for Media Engine and OslibMod ---------------------------------------------------------------------------------#
#- Booster & silverspring for EEPROM write/read functions --------------------------------------------------------------#
#- Akind for RemoteJoyLite ---------------------------------------------------------------------------------------------#
#- cooleyes for mpeg4 lib ----------------------------------------------------------------------------------------------#
#- Arshia001 for PSPAALib ----------------------------------------------------------------------------------------------#
#- InsertWittyName & MK2k for PGE sourcecode ---------------------------------------------------------------------------#
#- Youresam for LUA BMPLib ---------------------------------------------------------------------------------------------#
#- Raphael for vram manager code ---------------------------------------------------------------------------------------#
#- Dynodzzo for LSD concepts -------------------------------------------------------------------------------------------#
#- ab_portugal for Image.negative function -----------------------------------------------------------------------------#
#- JiC for drawCircle function ----------------------------------------------------------------------------------------#
#- Rapper_skull & DarkGiovy for testing LuaPlayer Plus and coming up with some neat ideas for it. ----------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------*/

#include "Timer.h"

#include <stdlib.h>

#ifdef DEBUG
extern int dwrite_output(const char *format, ...);
#endif

static clock_t getCMs(void) {
    return clock() / (u64)(CLOCKS_PER_SEC / 1000);
}

LPP_Timer *LPP_TimerCreate(clock_t startTime)
{
    LPP_Timer *timer = (LPP_Timer *)malloc(sizeof(LPP_Timer));
	if(!timer) {
	    #ifdef DEBUG
		dwrite_output("Function %s Line %d : Cannot allocate 'timer' to memory.\n", __FUNCTION__, __LINE__);
		#endif
		return null;
	}

	timer->measuredTime = getCMs();
	timer->offset = startTime;

	return(timer);
}

clock_t LPP_TimerStart(LPP_Timer *timer)
{
    if(timer->measuredTime) {
        return(getCMs() - timer->measuredTime + timer->offset);
    } else {
        timer->measuredTime = getCMs();
        return(timer->offset);
    }
}

clock_t LPP_TimerGetTime(LPP_Timer *timer)
{
    if(timer->measuredTime) {
        return(getCMs() - timer->measuredTime + timer->offset);
    } else {
        return(timer->offset);
    }
}

clock_t LPP_TimerStop(LPP_Timer *timer)
{
    if(timer->measuredTime) {
        timer->offset = getCMs() - timer->measuredTime + timer->offset;
        timer->measuredTime = (0);
    }

    return(timer->offset);
}

clock_t LPP_TimerReset(LPP_Timer *timer, clock_t offset)
{
    clock_t ret = 0;
    if(timer->measuredTime) {
        ret = getCMs() - timer->measuredTime + timer->offset;
    } else {
        ret = timer->offset;
    }

    timer->offset = offset;
    timer->measuredTime = 0;
    return(ret);
}

void LPP_TimerFree(LPP_Timer *timer)
{
    if(timer) free(timer);
}
