/*----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#------  This File is Part Of : ----------------------------------------------------------------------------------------#
#------- _  -------------------  ______   _   --------------------------------------------------------------------------#
#------ | | ------------------- (_____ \ | |  --------------------------------------------------------------------------#
#------ | | ---  _   _   ____    _____) )| |  ____  _   _   ____   ____   ----------------------------------------------#
#------ | | --- | | | | / _  |  |  ____/ | | / _  || | | | / _  ) / ___)  ----------------------------------------------#
#------ | |_____| |_| |( ( | |  | |      | |( ( | || |_| |( (/ / | |  --------------------------------------------------#
#------ |_______)\____| \_||_|  |_|      |_| \_||_| \__  | \____)|_|  --------------------------------------------------#
#------------------------------------------------- (____/  -------------------------------------------------------------#
#------------------------   ______   _   -------------------------------------------------------------------------------#
#------------------------  (_____ \ | |  -------------------------------------------------------------------------------#
#------------------------   _____) )| | _   _   ___   ------------------------------------------------------------------#
#------------------------  |  ____/ | || | | | /___)  ------------------------------------------------------------------#
#------------------------  | |      | || |_| ||___ |  ------------------------------------------------------------------#
#------------------------  |_|      |_| \____|(___/   ------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#- Licensed under the GPL License --------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#- Copyright (c) Nanni <cusunanni@hotmail.it> --------------------------------------------------------------------------#
#- Copyright (c) Rinnegatamante <rinnegatamante@eternalongju2.com> -----------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#- Official Forum : http://rinnegatamante.eu/luaplayerplus/forum.php ---------------------------------------------------#
#- For help using LuaPlayerPlus, coding help, and other please visit : http://rinnegatamante.eu/luaplayerplus/forum.php #
#-----------------------------------------------------------------------------------------------------------------------#
#- Credits : -----------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#- Homemister for LPHM sourcecode --------------------------------------------------------------------------------------#
#- Zack & Shine for LP Euphoria sourcecode -----------------------------------------------------------------------------#
#- ab5000 for support on psp-ita.com -----------------------------------------------------------------------------------#
#- valantin for sceIoMvdir and sceIoCpdir improved functions------------------------------------------------------------#
#- Dark_AleX for usbdevice ---------------------------------------------------------------------------------------------#
#- VirtuosFlame & ColdBird for iso drivers and kuBridge ----------------------------------------------------------------#
#- sakya for Media Engine and OslibMod ---------------------------------------------------------------------------------#
#- Booster & silverspring for EEPROM write/read functions --------------------------------------------------------------#
#- Akind for RemoteJoyLite ---------------------------------------------------------------------------------------------#
#- cooleyes for mpeg4 lib ----------------------------------------------------------------------------------------------#
#- Arshia001 for PSPAALib ----------------------------------------------------------------------------------------------#
#- InsertWittyName & MK2k for PGE sourcecode ---------------------------------------------------------------------------#
#- Youresam for LUA BMPLib ---------------------------------------------------------------------------------------------#
#- Raphael for vram manager code ---------------------------------------------------------------------------------------#
#- Dynodzzo for LSD concepts -------------------------------------------------------------------------------------------#
#- ab_portugal for Image.negative function -----------------------------------------------------------------------------#
#- JiC for drawCircle function ----------------------------------------------------------------------------------------#
#- Rapper_skull & DarkGiovy for testing LuaPlayer Plus and coming up with some neat ideas for it. ----------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------#
#-----------------------------------------------------------------------------------------------------------------------*/

#include "../LPP.h"
#include "../Libs/Timer/Timer.h"

UserdataStubs(Timer, LPP_Timer*);

static int luaTimer_new(lua_State *L) {
    Int16 args = lua_gettop(L);

    if(args > 1)
	{
	    return luaL_error(L, "Timer.new([startTime]) takes 1 or no arguments.");
	}

	LPP_Timer *t = LPP_TimerCreate(args == 1 ? luaL_checkint(L, 1) : 0);

	if(!t)
	{
	    return luaL_error(L, "Cannot create the timer.");
	}

	*pushTimer(L) = t;

	return(1);
}

static int luaTimer_start(lua_State *L)
{
    if(lua_gettop(L) != 1)
    {
        return luaL_error(L, "Timer:start() takes no arguments and must be called with a colon.");
    }
    lua_pushnumber(L, LPP_TimerStart(*toTimer(L, 1)));
    return(1);
}

static int luaTimer_getTime(lua_State *L) {
    if(lua_gettop(L) != 1)
	{
	    return luaL_error(L, "Timer:time() takes no arguments and must be called with a colon.");
	}
	lua_pushnumber(L, LPP_TimerGetTime(*toTimer(L, 1)));
	return(1);
}

static int luaTimer_stop(lua_State *L) {
    if(lua_gettop(L) != 1)
	{
	    return luaL_error(L, "Timer:stop() takes no arguments and must be called with a colon.");
	}
	lua_pushnumber(L, LPP_TimerStop(*toTimer(L, 1)));
	return(1);
}

static int luaTimer_reset(lua_State *L) {
    Int16 args = lua_gettop(L);
    if(args != 1 && args != 2)
	{
	    return luaL_error(L, "Timer:reset([startTime]) takes no arguments and must be called with a colon.");
	}
	lua_pushnumber(L, LPP_TimerReset(*toTimer(L, 1), args == 2 ? luaL_checkint(L, 2) : 0));
	return(1);
}

static int luaTimer__gc(lua_State *L) {
    LPP_TimerFree(*toTimer(L, 1));
	return(0);
}

static int luaTimer__tostring(lua_State *L) {
    lua_pushfstring(L, "%i", LPP_TimerGetTime(*toTimer(L, 1)));
	return(1);
}

static L_CONST luaL_reg luaTimer_methods[] = {
    { "new", luaTimer_new },
	{ "time", luaTimer_getTime },
	{ "start", luaTimer_start },
	{ "stop", luaTimer_stop },
	{ "reset", luaTimer_reset },
	{ 0, 0 }
};

static L_CONST luaL_reg luaTimer_meta[] = {
    { "__gc", luaTimer__gc },
	{ "__tostring", luaTimer__tostring },
	{ 0, 0 }
};

UserdataRegister(Timer, luaTimer_methods, luaTimer_meta);

void luaTimer_init(lua_State *L) {
    Timer_register(L);
}
